@@ -33,10 +33,6 @@
 #include "ibusserializable.h"
 #include "ibusconfig.h"
 
-#define DBUS_PATH_DBUS "/org/freedesktop/DBus"
-#define DBUS_SERVICE_DBUS "org.freedesktop.DBus"
-#define DBUS_INTERFACE_DBUS "org.freedesktop.DBus"
-
 #define IBUS_BUS_GET_PRIVATE(o)  \
    (G_TYPE_INSTANCE_GET_PRIVATE ((o), IBUS_TYPE_BUS, IBusBusPrivate))
 
@@ -48,6 +44,10 @@ enum {
     LAST_SIGNAL,
 };
 
+enum {
+    PROP_0 = 0,
+    PROP_CONNECT_ASYNC,
+};
 
 /* IBusBusPriv */
 struct _IBusBusPrivate {
@@ -59,6 +59,9 @@ struct _IBusBusPrivate {
     guint watch_ibus_signal_id;
     IBusConfig *config;
     gchar *unique_name;
+    gboolean connect_async;
+    gchar *bus_address;
+    GCancellable *cancellable;
 };
 
 static guint    bus_signals[LAST_SIGNAL] = { 0 };
@@ -93,6 +96,16 @@ static void      ibus_bus_call_async             (IBusBus                *bus,
                                                   GCancellable           *cancellable,
                                                   GAsyncReadyCallback     callback,
                                                   gpointer                user_data);
+static void      ibus_bus_set_property           (IBusBus                *bus,
+                                                  guint                   prop_id,
+                                                  const GValue           *value,
+                                                  GParamSpec             *pspec);
+static void      ibus_bus_get_property           (IBusBus                *bus,
+                                                  guint                   prop_id,
+                                                  GValue                 *value,
+                                                  GParamSpec             *pspec);
+
+static void     ibus_bus_close_connection        (IBusBus                *bus);
 
 G_DEFINE_TYPE (IBusBus, ibus_bus, IBUS_TYPE_OBJECT)
 
@@ -103,8 +116,25 @@ ibus_bus_class_init (IBusBusClass *class)
     IBusObjectClass *ibus_object_class = IBUS_OBJECT_CLASS (class);
 
     gobject_class->constructor = ibus_bus_constructor;
+    gobject_class->set_property = (GObjectSetPropertyFunc) ibus_bus_set_property;
+    gobject_class->get_property = (GObjectGetPropertyFunc) ibus_bus_get_property;
     ibus_object_class->destroy = ibus_bus_destroy;
 
+    /* install properties */
+    /**
+     * IBusBus:connect-async:
+     *
+     * Whether the #IBusBus object should connect asynchronously to the bus.
+     *
+     */
+    g_object_class_install_property (gobject_class,
+                                     PROP_CONNECT_ASYNC,
+                                     g_param_spec_boolean ("connect-async",
+                                                           "Connect Async",
+                                                           "Connect asynchronously to the bus",
+                                                           FALSE,
+                                                           G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+
     /* install signals */
     /**
      * IBusBus::connected:
@@ -244,59 +274,139 @@ _connection_closed_cb (GDBusConnection  *connection,
          * However we think the error message is almost harmless. */
         g_debug ("_connection_closed_cb: %s", error->message);
     }
+    ibus_bus_close_connection (bus);
+}
 
-    g_assert (bus->priv->connection == connection);
-    g_signal_handlers_disconnect_by_func (bus->priv->connection,
-                                          G_CALLBACK (_connection_closed_cb),
-                                          bus);
-    g_object_unref (bus->priv->connection);
-    bus->priv->connection = NULL;
-
+static void
+ibus_bus_close_connection (IBusBus *bus)
+{
     g_free (bus->priv->unique_name);
     bus->priv->unique_name = NULL;
 
     bus->priv->watch_dbus_signal_id = 0;
     bus->priv->watch_ibus_signal_id = 0;
 
-    g_signal_emit (bus, bus_signals[DISCONNECTED], 0);
-}
+    g_free (bus->priv->bus_address);
+    bus->priv->bus_address = NULL;
+
+    /* Cancel ongoing connect request. */
+    g_cancellable_cancel (bus->priv->cancellable);
+    g_cancellable_reset (bus->priv->cancellable);
 
-static void
-ibus_bus_connect (IBusBus *bus)
-{
     /* unref the old connection at first */
     if (bus->priv->connection != NULL) {
         g_signal_handlers_disconnect_by_func (bus->priv->connection,
                                               G_CALLBACK (_connection_closed_cb),
                                               bus);
+        if (!g_dbus_connection_is_closed(bus->priv->connection))
+            g_dbus_connection_close(bus->priv->connection, NULL, NULL, NULL);
         g_object_unref (bus->priv->connection);
         bus->priv->connection = NULL;
+        g_signal_emit (bus, bus_signals[DISCONNECTED], 0);
+    }
+}
+
+static void
+ibus_bus_connect_completed (IBusBus *bus)
+{
+    g_assert (bus->priv->connection);
+    /* FIXME */
+    ibus_bus_hello (bus);
+
+    g_signal_connect (bus->priv->connection,
+                      "closed",
+                      (GCallback) _connection_closed_cb,
+                      bus);
+    if (bus->priv->watch_dbus_signal) {
+        ibus_bus_watch_dbus_signal (bus);
+    }
+    if (bus->priv->watch_ibus_signal) {
+        ibus_bus_watch_ibus_signal (bus);
     }
 
-    if (ibus_get_address () != NULL) {
-        bus->priv->connection =
-            g_dbus_connection_new_for_address_sync (ibus_get_address (),
-                                                    G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT |
-                                                    G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
-                                                    NULL, NULL, NULL);
+    g_signal_emit (bus, bus_signals[CONNECTED], 0);
+}
+
+static void
+_bus_connect_async_cb (GObject      *source_object,
+                        GAsyncResult *res,
+                        gpointer      user_data)
+{
+    g_return_if_fail (user_data != NULL);
+    g_return_if_fail (IBUS_IS_BUS (user_data));
+
+    IBusBus *bus   = IBUS_BUS (user_data);
+    GError  *error = NULL;
+
+    bus->priv->connection =
+                g_dbus_connection_new_for_address_finish (res, &error);
+
+    if (error != NULL) {
+        g_warning ("Unable to connect to ibus: %s", error->message);
+        g_error_free (error);
+        error = NULL;
     }
 
+    if (bus->priv->connection != NULL) {
+        ibus_bus_connect_completed (bus);
+    }
+    else {
+        g_free (bus->priv->bus_address);
+        bus->priv->bus_address = NULL;
+    }
+
+    /* unref the ref from ibus_bus_connect */
+    g_object_unref (bus);
+}
+
+static void
+ibus_bus_connect_async (IBusBus *bus)
+{
+    const gchar *bus_address = ibus_get_address ();
+
+    if (bus_address == NULL)
+        return;
+
+    if (g_strcmp0 (bus->priv->bus_address, bus_address) == 0)
+        return;
+
+    /* Close current connection and cancel ongoing connect request. */
+    ibus_bus_close_connection (bus);
+
+    bus->priv->bus_address = g_strdup (bus_address);
+    g_object_ref (bus);
+    g_dbus_connection_new_for_address (
+            bus_address,
+            G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT |
+            G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
+            NULL,
+            bus->priv->cancellable,
+            _bus_connect_async_cb, bus);
+}
+
+static void
+ibus_bus_connect (IBusBus *bus)
+{
+    const gchar *bus_address = ibus_get_address ();
+
+    if (bus_address == NULL)
+        return;
+
+    if (g_strcmp0 (bus_address, bus->priv->bus_address) == 0 &&
+        bus->priv->connection != NULL)
+        return;
+
+    /* Close current connection and cancel ongoing connect request. */
+    ibus_bus_close_connection (bus);
+
+    bus->priv->connection = g_dbus_connection_new_for_address_sync (
+            bus_address,
+            G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT |
+            G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION,
+            NULL, NULL, NULL);
     if (bus->priv->connection) {
-        /* FIXME */
-        ibus_bus_hello (bus);
-
-        g_signal_connect (bus->priv->connection,
-                          "closed",
-                          (GCallback) _connection_closed_cb,
-                          bus);
-        if (bus->priv->watch_dbus_signal) {
-            ibus_bus_watch_dbus_signal (bus);
-        }
-        if (bus->priv->watch_ibus_signal) {
-            ibus_bus_watch_ibus_signal (bus);
-        }
-
-        g_signal_emit (bus, bus_signals[CONNECTED], 0);
+        bus->priv->bus_address = g_strdup (bus_address);
+        ibus_bus_connect_completed (bus);
     }
 }
 
@@ -312,10 +422,7 @@ _changed_cb (GFileMonitor       *monitor,
         event_type != G_FILE_MONITOR_EVENT_DELETED)
         return;
 
-    if (ibus_bus_is_connected (bus))
-        return;
-
-    ibus_bus_connect (bus);
+    ibus_bus_connect_async (bus);
 }
 
 static void
@@ -334,6 +441,9 @@ ibus_bus_init (IBusBus *bus)
     bus->priv->watch_ibus_signal = FALSE;
     bus->priv->watch_ibus_signal_id = 0;
     bus->priv->unique_name = NULL;
+    bus->priv->connect_async = FALSE;
+    bus->priv->bus_address = NULL;
+    bus->priv->cancellable = g_cancellable_new ();
 
     path = g_path_get_dirname (ibus_get_socket_path ());
 
@@ -347,8 +457,6 @@ ibus_bus_init (IBusBus *bus)
         }
     }
 
-    ibus_bus_connect (bus);
-
     file = g_file_new_for_path (ibus_get_socket_path ());
     bus->priv->monitor = g_file_monitor_file (file, 0, NULL, NULL);
 
@@ -358,6 +466,36 @@ ibus_bus_init (IBusBus *bus)
     g_free (path);
 }
 
+static void
+ibus_bus_set_property (IBusBus      *bus,
+                       guint         prop_id,
+                       const GValue *value,
+                       GParamSpec   *pspec)
+{
+    switch (prop_id) {
+    case PROP_CONNECT_ASYNC:
+        bus->priv->connect_async = g_value_get_boolean (value);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (bus, prop_id, pspec);
+    }
+}
+
+static void
+ibus_bus_get_property (IBusBus    *bus,
+                       guint       prop_id,
+                       GValue     *value,
+                       GParamSpec *pspec)
+{
+    switch (prop_id) {
+    case PROP_CONNECT_ASYNC:
+        g_value_set_boolean (value, bus->priv->connect_async);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (bus, prop_id, pspec);
+    }
+}
+
 static GObject*
 ibus_bus_constructor (GType                  type,
                       guint                  n_params,
@@ -371,6 +509,11 @@ ibus_bus_constructor (GType                  type,
         /* make bus object sink */
         g_object_ref_sink (object);
         _bus = IBUS_BUS (object);
+
+        if (_bus->priv->connect_async)
+            ibus_bus_connect_async (_bus);
+        else
+            ibus_bus_connect (_bus);
     }
     else {
         object = g_object_ref (_bus);
@@ -410,6 +553,13 @@ ibus_bus_destroy (IBusObject *object)
     g_free (bus->priv->unique_name);
     bus->priv->unique_name = NULL;
 
+    g_free (bus->priv->bus_address);
+    bus->priv->bus_address = NULL;
+
+    g_cancellable_cancel (bus->priv->cancellable);
+    g_object_unref (bus->priv->cancellable);
+    bus->priv->cancellable = NULL;
+
     IBUS_OBJECT_CLASS (ibus_bus_parent_class)->destroy (object);
 }
 
@@ -484,11 +634,22 @@ _async_finish_guint (GAsyncResult *res,
 IBusBus *
 ibus_bus_new (void)
 {
-    IBusBus *bus = IBUS_BUS (g_object_new (IBUS_TYPE_BUS, NULL));
+    IBusBus *bus = IBUS_BUS (g_object_new (IBUS_TYPE_BUS,
+                                           "connect-async", FALSE,
+                                           NULL));
 
     return bus;
 }
 
+IBusBus *
+ibus_bus_new_async (void)
+{
+    IBusBus *bus = IBUS_BUS (g_object_new (IBUS_TYPE_BUS,
+                                           "connect-async", TRUE,
+                                           NULL));
+
+    return bus;
+}
 
 gboolean
 ibus_bus_is_connected (IBusBus *bus)
@@ -501,7 +662,6 @@ ibus_bus_is_connected (IBusBus *bus)
     return TRUE;
 }
 
-
 IBusInputContext *
 ibus_bus_create_input_context (IBusBus      *bus,
                                const gchar  *client_name)
@@ -618,7 +778,7 @@ ibus_bus_create_input_context_async (IBusBus            *bus,
 
     if (cancellable != NULL) {
         g_object_set_data_full ((GObject *)simple,
-                                "concellable",
+                                "cancellable",
                                 g_object_ref (cancellable),
                                 (GDestroyNotify)g_object_unref);
     }
@@ -1381,7 +1541,9 @@ ibus_bus_do_list_engines (IBusBus *bus, gboolean active_engines_only)
         g_variant_get (result, "(av)", &iter);
         GVariant *var;
         while (g_variant_iter_loop (iter, "v", &var)) {
-            retval = g_list_append (retval, ibus_serializable_deserialize (var));
+            IBusSerializable *serializable = ibus_serializable_deserialize (var);
+            g_object_ref_sink (serializable);
+            retval = g_list_append (retval, serializable);
         }
         g_variant_iter_free (iter);
         g_variant_unref (result);
@@ -1435,7 +1597,9 @@ ibus_bus_list_engines_async_finish (IBusBus      *bus,
     g_variant_get (variant, "(av)", &iter);
     GVariant *var;
     while (g_variant_iter_loop (iter, "v", &var)) {
-        retval = g_list_append (retval, ibus_serializable_deserialize (var));
+        IBusSerializable *serializable = ibus_serializable_deserialize (var);
+        g_object_ref_sink (serializable);
+        retval = g_list_append (retval, serializable);
     }
     g_variant_iter_free (iter);
     return retval;
@@ -1478,6 +1642,38 @@ ibus_bus_list_active_engines_async_finish (IBusBus      *bus,
     return ibus_bus_list_engines_async_finish (bus, res, error);
 }
 
+IBusEngineDesc **
+ibus_bus_get_engines_by_names (IBusBus             *bus,
+                               const gchar * const *names)
+{
+    g_return_val_if_fail (IBUS_IS_BUS (bus), NULL);
+
+    GVariant *result;
+    result = ibus_bus_call_sync (bus,
+                                 IBUS_SERVICE_IBUS,
+                                 IBUS_PATH_IBUS,
+                                 IBUS_INTERFACE_IBUS,
+                                 "GetEnginesByNames",
+                                 g_variant_new("(^as)", names),
+                                 G_VARIANT_TYPE ("(av)"));
+    if (result == NULL)
+        return NULL;
+
+    GArray *array = g_array_new (TRUE, TRUE, sizeof (IBusEngineDesc *));
+    GVariantIter *iter = NULL;
+    g_variant_get (result, "(av)", &iter);
+    GVariant *var;
+    while (g_variant_iter_loop (iter, "v", &var)) {
+        IBusEngineDesc *desc = (IBusEngineDesc *) ibus_serializable_deserialize (var);
+        g_object_ref_sink (desc);
+        g_array_append_val (array, desc);
+    }
+    g_variant_iter_free (iter);
+    g_variant_unref (result);
+
+    return (IBusEngineDesc **)g_array_free (array, FALSE);
+}
+
 static void
 _config_destroy_cb (IBusConfig *config,
                     IBusBus    *bus)
